﻿using System;
using System.Text;
using System.Globalization;
using HeuristicLab.Persistence.Interfaces;

namespace HeuristicLab.Persistence {

  public class XmlFormat : IFormat {
    public string Name { get { return "XML"; } }
    public static readonly XmlFormat Instance = new XmlFormat();    
  }

  public class String2XmlFormatter : IFormatter {
    public Type Type { get { return typeof(string); } }
    public IFormat Format { get { return XmlFormat.Instance;  } }
    public object Serialize(object o) {      
      return "<![CDATA[" +
        ((string)o).Replace("]]>", "]]]]><![CDATA[>") +
        "]]>";
    }
    public object DeSerialize(object o) {
      StringBuilder sb = new StringBuilder();
      foreach (string s in ((string)o).Split(
        new[] { "<![CDATA[", "]]>" },
        StringSplitOptions.RemoveEmptyEntries)) {
        sb.Append(s);
      }
      return sb.ToString();
    }
  }

  public class Int2XmlFormatter : IFormatter {
    public Type Type { get { return typeof(int); } }
    public IFormat Format { get { return XmlFormat.Instance; } }
    public object Serialize(object o) {
      return ((int)o).ToString();
    }
    public object DeSerialize(object o) {
      return int.Parse((string)o);
    }
  }

  public class Double2XmlFormatter : IFormatter {    
    public Type Type { get { return typeof(double); } }
    public IFormat Format { get { return XmlFormat.Instance; } }
    public object Serialize(object o) {
      return ((double)o).ToString("r", CultureInfo.InvariantCulture);      
    }
    public object DeSerialize(object o) {
      return double.Parse((string)o, CultureInfo.InvariantCulture);
    }
  }

  public class Boolean2XmlFormatter : IFormatter {    
    public Type Type { get { return typeof(bool); } }
    public IFormat Format { get { return XmlFormat.Instance; } }
    public object Serialize(object o) {
      return ((bool)o).ToString();
    }
    public object DeSerialize(object o) {
      return bool.Parse((string)o);
    }
  }

  public class DateTime2XmlFormatter : IFormatter {    
    public Type Type { get { return typeof(DateTime); } }
    public IFormat Format { get { return XmlFormat.Instance; } }  
    public object Serialize(object o) {
      return ((DateTime)o).Ticks.ToString();      
    }
    public object DeSerialize(object o) {
      return new DateTime(long.Parse((string)o));
    }
  }

  
}